/*
 * <sys/capability.h>
 *
 * 
 * Copyright (C) 1997   Aleph One
 * Copyright (C) 1997-8 Andrew G. Morgan <morgan@linux.kernel.org>
 *
 * POSIX.1e Draft Standard 17: Capabilities           <sys/capability.h>
 */

#ifndef _SYS_CAPABILITY_H
#define _SYS_CAPABILITY_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * This file complements the kernel file by providing prototype
 * information for the user library.
 */

#include <sys/types.h>
/*
 * This is <linux/capability.h>
 *
 * Andrew G. Morgan <morgan@kernel.org>
 * Alexander Kjeldaas <astor@guardian.no>
 * with help from Aleph1, Roland Buresund and Andrew Main.
 *
 * See here for the libcap library ("POSIX.1e" DS17 compliance):
 *
 *   http://www.kernel.org/pub/linux/libs/security/linux-privs/
 */ 

#ifndef _LINUX_CAPABILITY_H
#define _LINUX_CAPABILITY_H

#include <linux/types.h>

/* User-level do most of the mapping between kernel and user
   capabilities based on the version tag given by the kernel. The
   kernel might be somewhat backwards compatible, but don't bet on
   it. */

/* XXX - Note, cap_t, is defined by POSIX to be an "opaque" pointer to
   a set of three capability sets.  The transposition of 3*the
   following structure to such a composite is better handled in a user
   library since the draft standard requires the use of malloc/free
   etc.. */
 
#define _LINUX_CAPABILITY_VERSION_0  0x19980330
#define _LINUX_U32S_IN_CAPS_0        0x1

#define _LINUX_CAPABILITY_VERSION_1  0x20000603
#define _LINUX_U32S_IN_CAPS_1        0x1

#define _LINUX_CAPABILITY_VERSION_2  0x20010113
#define _LINUX_U32S_IN_CAPS_2        0x2

/* current */
#define _LINUX_CAPABILITY_VERSION    _LINUX_CAPABILITY_VERSION_2
#define _LINUX_U32S_IN_CAPS          _LINUX_U32S_IN_CAPS_2

/* Legacy support */

typedef struct __user_cap_header_0_struct {
	__u32 version;
	int pid;
} *cap_user_header_0_t;

typedef struct __user_cap_data_0_struct {
	__u32 effective;
	__u32 permitted;
	__u32 inheritable;
} *cap_user_data_0_t;

/* Current definition */

typedef struct __user_cap_header_struct {
	__u32 version;
	int type;
	union {
		int pid;
		int fd;
		const char *file;
	} u;
} *cap_user_header_t;

/* type definitions */

#define CAP_USERHEADER_PID       0x00000001
#define CAP_USERHEADER_FILEDES   0x00000002
#define CAP_USERHEADER_FILE      0x00000003

/* #define CAP_USERHEADER_FSMASK    0x00000004 */

#define CAP_USERHEADER_X_WITH_E  0x00000005
#define CAP_USERHEADER_X_WITH_I  0x00000006
#define CAP_USERHEADER_X_WITH_P  0x00000007

typedef struct __user_cap_data_struct {
        __u32 effective[_LINUX_U32S_IN_CAPS];
        __u32 permitted[_LINUX_U32S_IN_CAPS];
        __u32 inheritable[_LINUX_U32S_IN_CAPS];
} *cap_user_data_t;

#ifdef __KERNEL__

typedef struct {
    __u32 chunk[_LINUX_U32S_IN_CAPS];
} kernel_cap_t;

typedef struct __kernel_cap_data_struct {
	kernel_cap_t effective;
	kernel_cap_t permitted;
	kernel_cap_t inheritable;
} *cap_kernel_data_t;

#endif /* __KERNEL__ */


/**
 ** POSIX-draft defined capabilities. 
 **/

/* In a system with the [_POSIX_CHOWN_RESTRICTED] option defined, this
   overrides the restriction of changing file ownership and group
   ownership. */

#define CAP_CHOWN            0

/* Override all DAC access, including ACL execute access if
   [_POSIX_ACL] is defined. Excluding DAC access covered by
   CAP_LINUX_IMMUTABLE. */

#define CAP_DAC_OVERRIDE     1

/* Overrides all DAC restrictions regarding read and search on files
   and directories, including ACL restrictions if [_POSIX_ACL] is
   defined. Excluding DAC access covered by CAP_LINUX_IMMUTABLE. */

#define CAP_DAC_READ_SEARCH  2
    
/* Overrides all restrictions about allowed operations on files, where
   file owner ID must be equal to the user ID, except where CAP_FSETID
   is applicable. It doesn't override MAC and DAC restrictions. */

#define CAP_FOWNER           3

/* Overrides the following restrictions that the effective user ID
   shall match the file owner ID when setting the S_ISUID and S_ISGID
   bits on that file; that the effective group ID (or one of the
   supplementary group IDs) shall match the file owner ID when setting
   the S_ISGID bit on that file; that the S_ISUID and S_ISGID bits are
   cleared on successful return from chown(2) (not implemented). */

#define CAP_FSETID           4

/* Overrides the restriction that the real or effective user ID of a
   process sending a signal must match the real or effective user ID
   of the process receiving the signal. */

#define CAP_KILL             5

/* Allows setgid(2) manipulation */
/* Allows setgroups(2) */
/* Allows forged gids on socket credentials passing. */

#define CAP_SETGID           6

/* Allows set*uid(2) manipulation (including fsuid). */
/* Allows forged pids on socket credentials passing. */

#define CAP_SETUID           7


/**
 ** Linux-specific capabilities
 **/

/* Arbitrarily raise any capability in the current process' Inheritable set
 *    (that isn't precluded by the local cap_bset).
 * [ XXX placeholder - permit process to turn off superuser support. ]
 * [ XXX placeholder - permit process to turn off setuid0 support. ]
 */

#define CAP_SETPCAP          8

/* Allow modification of S_IMMUTABLE and S_APPEND file attributes */

#define CAP_LINUX_IMMUTABLE  9

/* Allows binding to TCP/UDP sockets below 1024 */
/* Allows binding to ATM VCIs below 32 */

#define CAP_NET_BIND_SERVICE 10

/* Allow broadcasting, listen to multicast */

#define CAP_NET_BROADCAST    11

/* Allow interface configuration */
/* Allow administration of IP firewall, masquerading and accounting */
/* Allow setting debug option on sockets */
/* Allow modification of routing tables */
/* Allow setting arbitrary process / process group ownership on
   sockets */
/* Allow binding to any address for transparent proxying */
/* Allow setting TOS (type of service) */
/* Allow setting promiscuous mode */
/* Allow clearing driver statistics */
/* Allow multicasting */
/* Allow read/write of device-specific registers */
/* Allow activation of ATM control sockets */

#define CAP_NET_ADMIN        12

/* Allow use of RAW sockets */
/* Allow use of PACKET sockets */

#define CAP_NET_RAW          13

/* Allow locking of shared memory segments */
/* Allow mlock and mlockall (which doesn't really have anything to do
   with IPC) */

#define CAP_IPC_LOCK         14

/* Override IPC ownership checks */

#define CAP_IPC_OWNER        15

/* Insert and remove kernel modules - modify kernel without limit */
/* Modify current->cap_bset */
#define CAP_SYS_MODULE       16

/* Allow ioperm/iopl access */
/* Allow sending USB messages to any device via /proc/bus/usb */

#define CAP_SYS_RAWIO        17

/* Allow use of chroot() */

#define CAP_SYS_CHROOT       18

/* Allow ptrace() of any process */

#define CAP_SYS_PTRACE       19

/* Allow configuration of process accounting */

#define CAP_SYS_PACCT        20

/* Allow configuration of the secure attention key */
/* Allow administration of the random device */
/* Allow examination and configuration of disk quotas */
/* Allow configuring the kernel's syslog (printk behaviour) */
/* Allow setting the domainname */
/* Allow setting the hostname */
/* Allow calling bdflush() */
/* Allow mount() and umount(), setting up new smb connection */
/* Allow some autofs root ioctls */
/* Allow nfsservctl */
/* Allow VM86_REQUEST_IRQ */
/* Allow to read/write pci config on alpha */
/* Allow irix_prctl on mips (setstacksize) */
/* Allow flushing all cache on m68k (sys_cacheflush) */
/* Allow removing semaphores */
/* Used instead of CAP_CHOWN to "chown" IPC message queues, semaphores
   and shared memory */
/* Allow locking/unlocking of shared memory segment */
/* Allow turning swap on/off */
/* Allow forged pids on socket credentials passing */
/* Allow setting readahead and flushing buffers on block devices */
/* Allow setting geometry in floppy driver */
/* Allow turning DMA on/off in xd driver */
/* Allow administration of md devices (mostly the above, but some
   extra ioctls) */
/* Allow tuning the ide driver */
/* Allow access to the nvram device */
/* Allow administration of apm_bios, serial and bttv (TV) device */
/* Allow manufacturer commands in isdn CAPI support driver */
/* Allow reading non-standardized portions of pci configuration space */
/* Allow DDI debug ioctl on sbpcd driver */
/* Allow setting up serial ports */
/* Allow sending raw qic-117 commands */
/* Allow enabling/disabling tagged queuing on SCSI controllers and sending
   arbitrary SCSI commands */
/* Allow setting encryption key on loopback filesystem */

#define CAP_SYS_ADMIN        21

/* Allow use of reboot() */

#define CAP_SYS_BOOT         22

/* Allow raising priority and setting priority on other (different
   UID) processes */
/* Allow use of FIFO and round-robin (realtime) scheduling on own
   processes and setting the scheduling algorithm used by another
   process. */

#define CAP_SYS_NICE         23

/* Override resource limits. Set resource limits. */
/* Override quota limits. */
/* Override reserved space on ext2 filesystem */
/* NOTE: ext2 honors fsuid when checking for resource overrides, so 
   you can override using fsuid too */
/* Override size restrictions on IPC message queues */
/* Allow more than 64hz interrupts from the real-time clock */
/* Override max number of consoles on console allocation */
/* Override max number of keymaps */

#define CAP_SYS_RESOURCE     24

/* Allow manipulation of system clock */
/* Allow irix_stime on mips */
/* Allow setting the real-time clock */

#define CAP_SYS_TIME         25

/* Allow configuration of tty devices */
/* Allow vhangup() of tty */

#define CAP_SYS_TTY_CONFIG   26

/* Allow the privileged aspects of mknod() */

#define CAP_MKNOD            27

/* Allow taking of leases on files */

#define CAP_LEASE            28

/* Allow a process to set file capabilities */

#define CAP_SETFCAP          32

#ifdef __KERNEL__

/*
 * Internal kernel functions only
 */
 
#define to_cap_t(x)          ((kernel_cap_t) { {(x), (x)} })
#define cap_t(x)             (x)

#define CAP_EMPTY_SET        to_cap_t(0)
#define CAP_FULL_SET         to_cap_t(~0)
#define CAP_INIT_EFF_SET     CAP_FULL_SET
#define CAP_INIT_INH_SET     to_cap_t(0)

#define CAP_TO_SUBMASK(x)    (1 << (x & 31))
#define CAP_TO_INDEX(x)      ((x) / 32)

#define cap_raise(c, flag)   \
       ((c).chunk[CAP_TO_INDEX(flag)] |=  CAP_TO_SUBMASK(flag))
#define cap_lower(c, flag)   \
       ((c).chunk[CAP_TO_INDEX(flag)] &= ~CAP_TO_SUBMASK(flag))
#define cap_raised(c, flag)  \
       ((c).chunk[CAP_TO_INDEX(flag)] &   CAP_TO_SUBMASK(flag))

static inline kernel_cap_t cap_combine(kernel_cap_t a, kernel_cap_t b)
{
     kernel_cap_t dest;
     int i;
     for (i=0; i<_LINUX_U32S_IN_CAPS; ++i) {
	 dest.chunk[i] = a.chunk[i] | b.chunk[i];
     }
     return dest;
}

static inline kernel_cap_t cap_intersect(kernel_cap_t a, kernel_cap_t b)
{
     kernel_cap_t dest;
     int i;
     for (i=0; i<_LINUX_U32S_IN_CAPS; ++i) {
	 dest.chunk[i] = a.chunk[i] & b.chunk[i];
     }
     return dest;
}

static inline kernel_cap_t cap_drop(kernel_cap_t a, kernel_cap_t drop)
{
     kernel_cap_t dest;
     int i;
     for (i=0; i<_LINUX_U32S_IN_CAPS; ++i) {
	 dest.chunk[i] = a.chunk[i] & ~drop.chunk[i];
     }
     return dest;
}

static inline kernel_cap_t cap_invert(kernel_cap_t c)
{
     kernel_cap_t dest;
     int i;
     for (i=0; i<_LINUX_U32S_IN_CAPS; ++i) {
	 dest.chunk[i] = ~c.chunk[i];
     }
     return dest;
}

#if _LINUX_U32S_IN_CAPS == 2
# define cap_isclear(c)       (!((c).chunk[0]|(c).chunk[1]))
# define cap_issubset(a,set)  ((!((a).chunk[0] & ~(set).chunk[0])) \
                               && (!((a).chunk[1] & ~(set).chunk[1])))
#else
# error \
   "_LINUX_U32S_IN_CAPS is not 2 - please fix cap_isclear and cap_issubset"
#endif

#define __CAP_RUN_OVER_ARRAY(x) \
do { \
    int __cindex; \
    for (__cindex=0; __cindex<_LINUX_U32S_IN_CAPS; ++__cindex) { \
	x ; \
    } \
} while(0)
        
#define cap_clear(c)        __CAP_RUN_OVER_ARRAY(c.chunk[__cindex] = 0)
#define cap_set_full(c)     __CAP_RUN_OVER_ARRAY(c.chunk[__cindex] = ~0)
#define cap_mask(c,mask)    \
           __CAP_RUN_OVER_ARRAY(c.chunk[__cindex] &= mask.chunk[__cindex])

/* Used to decide between falling back on the old suser() or fsuser().
 * NOTE: this is a pre-computed mask and is assumed to only use bits
 *       in the low 32 bits of the capability set. If this assumption
 *       becomes invalid, then we'll need to modify "cap_is_fs_cap(x)"
 *       below.
 */

#define CAP_FS_MASK           0x1f
#define CAP_NFSD_MASK         (CAP_FS_MASK | CAP_TO_SUBMASK(CAP_SYS_RESOURCE))
#define cap_is_fs_cap(x)      (CAP_TO_SUBMASK(x) & CAP_FS_MASK)
#define cap_is_nfsd_cap(x)    (CAP_TO_SUBMASK(x) & CAP_NFSD_MASK)

#define cap_drop_fs_caps(cap) \
        do { (cap).chunk[0] &= ~CAP_FS_MASK; } while (0)
#define cap_drop_nfsd_caps(cap) \
        do { (cap).chunk[0] &= ~CAP_NFSD_MASK; } while (0)
#define cap_raise_permitted_fs_caps() \
        do { current->cap_effective.chunk[0] |= \
                 CAP_FS_MASK & current->cap_permitted.chunk[0]; } while (0)
#define cap_raise_permitted_nfsd_caps() \
        do { current->cap_effective.chunk[0] |= \
                 CAP_NFSD_MASK & current->cap_permitted.chunk[0]; } while (0)

/* functions defined in kernel/capability.c */

int cap_copy_from_user(cap_kernel_data_t cap, const void *dataptr,
		       __u32 version, cap_user_header_t header);
int cap_copy_to_user(const cap_kernel_data_t data, void *dataptr,
		     __u32 version);

int capsetp(int pid, struct __kernel_cap_data_struct *caps, __u32 version);
int capgetp(int pid, void *dataptr, __u32 version);
int capsetx(const struct __kernel_cap_data_struct *cap, int which_cap);
int capgetx(struct __user_cap_data_struct *cap, int which_cap, __u32 version);

#endif /* __KERNEL__ */

#endif /* !_LINUX_CAPABILITY_H */

/*
 * POSIX capability types
 */

/*
 * Opaque capability handle (defined internally by libcap)
 * internal capability representation
 */
typedef struct _cap_struct *cap_t;

/* "external" capability representation is a (void *) */

/*
 * This is the type used to identify capabilities
 */

typedef int cap_value_t;

/*
 * Set identifiers
 */
typedef enum {
    CAP_EFFECTIVE=0,                        /* Specifies the effective flag */
    CAP_PERMITTED=1,                        /* Specifies the permitted flag */
    CAP_INHERITABLE=2                     /* Specifies the inheritable flag */
} cap_flag_t;

/*
 * These are the states available to each capability
 */
typedef enum {
    CAP_CLEAR=0,                            /* The flag is cleared/disabled */
    CAP_SET=1                                    /* The flag is set/enabled */
} cap_flag_value_t;

/*
 * User-space capability manipulation routines
 */

/* libcap/cap_alloc.c */
cap_t   cap_dup(cap_t);
int     cap_free(void *);
cap_t   cap_init(void);

/* libcap/cap_flag.c */
int     cap_get_flag(cap_t, cap_value_t, cap_flag_t, cap_flag_value_t *);
int     cap_set_flag(cap_t, cap_flag_t, int, cap_value_t *, cap_flag_value_t);
int     cap_clear(cap_t);

/* libcap/cap_file.c */
cap_t   cap_get_fd(int);
cap_t   cap_get_file(const char *);
int     cap_set_fd(int, cap_t);
int     cap_set_file(const char *, cap_t);

/* libcap/cap_proc.c */
cap_t   cap_get_proc(void);
int     cap_set_proc(cap_t);

cap_t   cap_get_bound(cap_flag_t);
int     cap_set_bound(cap_t, cap_flag_t);

/* libcap/cap_extint.c */
ssize_t cap_size(cap_t);
ssize_t cap_copy_ext(void *, cap_t, ssize_t);
cap_t   cap_copy_int(const void *);

/* libcap/cap_text.c */
cap_t   cap_from_text(const char *);
char *  cap_to_text(cap_t, ssize_t *);

/*
 * Linux capability system calls: defined in libcap but only available
 * if the following _POSIX_SOURCE is _undefined_
 */

#if !defined(_POSIX_SOURCE)

extern int capset(cap_user_header_t header, cap_user_data_t data);
extern int capget(cap_user_header_t header, const cap_user_data_t data);
extern int capgetp(pid_t pid, cap_t cap_d);
extern char const *_cap_names[];

#endif /* !defined(_POSIX_SOURCE) */

#ifdef __cplusplus
}
#endif

#endif /* _SYS_CAPABILITY_H */
